/* This file is part of MAUS: http://micewww.pp.rl.ac.uk:8080/projects/maus
 *
 * MAUS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MAUS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MAUS.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _MAUS_INPUTCPPREALDATA_CABLINGTOOLS_EMR_HH_
#define _MAUS_INPUTCPPREALDATA_CABLINGTOOLS_EMR_HH_


#include <stdint.h>
#include <stdlib.h>
#include <limits.h>
#include <string>
#include <vector>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <fstream>


#include "DAQChannelMap.hh"
#include "Squeal.hh"
#include "Squeak.hh"

using std::string;
using std::ostream;
using std::istream;
using std::ifstream;
using std::stringstream;

namespace MAUS {
//////////////////////////////////////////////////////////////////////////////////////////////
/** Identifier for a single EMR channel.
 * This class is used to hold and manage all the information needed
 * to identifiy one channel in the EMR detectors.
 */
class EMRChannelKey {
 public:

  EMRChannelKey()
  :_module(-999), _plane(-999), _bar(-999), _detector("unknown") {}

  EMRChannelKey(int m, int pl, int b, string d)
  :_module(m), _plane(pl), _bar(b), _detector(d) {}

  explicit EMRChannelKey(string keyStr) throw(Squeal);
  virtual ~EMRChannelKey() {}

  bool operator==( EMRChannelKey key );
  bool operator!=( EMRChannelKey key );

  friend ostream& operator<<( ostream& stream, EMRChannelKey key );
  friend istream& operator>>( istream& stream, EMRChannelKey &key ) throw(Squeal);

  /** This function converts the DAQChannelKey into string.
  * \return String identifier.
  */
  string str();

  int module() const {return _module;}
  int plane()  const {return _plane;}
  int bar()    const {return _bar;}
  string detector() const {return _detector;}

  void SetModule(int xModule) {_module = xModule;}
  void SetPlane(int xPlane)   {_plane = xPlane;}
  void SetBar(int xBar)       {_bar = xBar;}
  void SetDetector(string xDetector) {_detector = xDetector;}

 private:

  /// Module number.
  int _module;

  /// Plane number.
  int _plane;

  /// Bar number.
  int _bar;

  /// Name of the detector.
  string _detector;
};

//////////////////////////////////////////////////////////////////////////////////////////////
/** Complete map of all EMR channels.
 * This class is used to hold and manage the informatin for all EMR Channel.
 */
class EMRChannelMap {
 public:

  EMRChannelMap() {}
  virtual ~EMRChannelMap();

  /// Initialize the map from text file.
  bool InitFromFile(string filename);

  /// Not implemented.
  void InitFromCDB();

 /** Return pointer to the EMR key.
 * This function returns pointer to the EMR channel key for the required DAQ channel.
 * \param[in] daqch DAQ channel to search for.
 * \return The key of the EMR channel connected to the given DAQ channel.
 */
  EMRChannelKey* find(DAQChannelKey* daqKey);

 /** Return pointer to the EMR key.
 * This function returns pointer to the EMR channel key for the required DAQ channel.
 * \param[in] daqch DAQ channel to search for, coded as string.
 * \return The key of the EMR channel connected to the given DAQ channel.
 */
  EMRChannelKey* find(string daqKeyStr);

 private:

  std::vector<EMRChannelKey*>   _emrKey;
  std::vector<DAQChannelKey*>   _dbbKey;
};
}
#endif
// 4 17 46 58 





